package com.sun.media.sound;

import java.lang.ref.WeakReference;
import javax.sound.midi.Transmitter;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.AudioSystem;
import javax.sound.midi.MidiUnavailableException;
import java.util.StringTokenizer;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import java.util.Properties;
import java.util.Arrays;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import javax.sound.midi.MidiSystem;
import java.io.BufferedInputStream;
import java.security.AccessController;
import java.io.FileInputStream;
import java.io.File;
import java.io.InputStream;
import java.security.PrivilegedAction;
import javax.sound.midi.VoiceStatus;
import javax.sound.midi.MidiChannel;
import javax.sound.midi.Patch;
import javax.sound.midi.Instrument;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.HashMap;
import javax.sound.midi.Receiver;
import java.util.ArrayList;
import java.util.Map;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.midi.Soundbank;
import javax.sound.sampled.SourceDataLine;
import javax.sound.midi.MidiDevice;

public final class SoftSynthesizer implements AudioSynthesizer, ReferenceCountingDevice
{
    static final String INFO_NAME = "Gervill";
    static final String INFO_VENDOR = "OpenJDK";
    static final String INFO_DESCRIPTION = "Software MIDI Synthesizer";
    static final String INFO_VERSION = "1.0";
    static final MidiDevice.Info info;
    private static SourceDataLine testline;
    private static Soundbank defaultSoundBank;
    SoftSynthesizer.WeakAudioStream weakstream;
    final Object control_mutex;
    int voiceIDCounter;
    int voice_allocation_mode;
    boolean load_default_soundbank;
    boolean reverb_light;
    boolean reverb_on;
    boolean chorus_on;
    boolean agc_on;
    SoftChannel[] channels;
    SoftChannelProxy[] external_channels;
    private boolean largemode;
    private int gmmode;
    private int deviceid;
    private AudioFormat format;
    private SourceDataLine sourceDataLine;
    private SoftAudioPusher pusher;
    private AudioInputStream pusher_stream;
    private float controlrate;
    private boolean open;
    private boolean implicitOpen;
    private String resamplerType;
    private SoftResampler resampler;
    private int number_of_midi_channels;
    private int maxpoly;
    private long latency;
    private boolean jitter_correction;
    private SoftMainMixer mainmixer;
    private SoftVoice[] voices;
    private Map<String, SoftTuning> tunings;
    private Map<String, SoftInstrument> inslist;
    private Map<String, ModelInstrument> loadedlist;
    private ArrayList<Receiver> recvslist;
    
    public SoftSynthesizer() {
        this.weakstream = null;
        this.control_mutex = this;
        this.voiceIDCounter = 0;
        this.voice_allocation_mode = 0;
        this.load_default_soundbank = false;
        this.reverb_light = true;
        this.reverb_on = true;
        this.chorus_on = true;
        this.agc_on = true;
        this.external_channels = null;
        this.largemode = false;
        this.gmmode = 0;
        this.deviceid = 0;
        this.format = new AudioFormat(44100.0f, 16, 2, true, false);
        this.sourceDataLine = null;
        this.pusher = null;
        this.pusher_stream = null;
        this.controlrate = 147.0f;
        this.open = false;
        this.implicitOpen = false;
        this.resamplerType = "linear";
        this.resampler = new SoftLinearResampler();
        this.number_of_midi_channels = 16;
        this.maxpoly = 64;
        this.latency = 200000L;
        this.jitter_correction = false;
        this.tunings = new HashMap();
        this.inslist = new HashMap();
        this.loadedlist = new HashMap();
        this.recvslist = new ArrayList();
    }
    
    private void getBuffers(ModelInstrument modelInstrument, List<ModelByteBuffer> list) {
        for (ModelPerformer modelPerformer : modelInstrument.getPerformers()) {
            if (modelPerformer.getOscillators() != null) {
                for (ModelOscillator modelOscillator : modelPerformer.getOscillators()) {
                    if ((modelOscillator instanceof ModelByteBufferWavetable)) {
                        ModelByteBufferWavetable modelByteBufferWavetable = (ModelByteBufferWavetable)modelOscillator;
                        ModelByteBuffer buffer = modelByteBufferWavetable.getBuffer();
                        if (buffer != null) {
                            list.add(buffer);
                        }
                        ModelByteBuffer get8BitExtensionBuffer = modelByteBufferWavetable.get8BitExtensionBuffer();
                        if (get8BitExtensionBuffer != null) {
                            list.add(get8BitExtensionBuffer);
                        }
                        else {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
    
    private boolean loadSamples(List<ModelInstrument> list) {
        if (this.largemode) {
            return true;
        }
        ArrayList<ModelByteBuffer> list2 = new ArrayList();
        Iterator<Object> iterator = list.iterator();
        while (iterator.hasNext()) {
            this.getBuffers((ModelInstrument)iterator.next(), list2);
        }
        try {
            ModelByteBuffer.loadAll(list2);
        }
        catch (IOException ex) {
            return false;
        }
        return true;
    }
    
    private boolean loadInstruments(List<ModelInstrument> list) {
        if (!this.isOpen()) {
            return false;
        }
        if (!this.loadSamples(list)) {
            return false;
        }
        synchronized (this.control_mutex) {
            if (this.channels != null) {
                for (SoftChannel softChannel : this.channels) {
                    softChannel.current_instrument = null;
                    softChannel.current_director = null;
                }
            }
            for (Instrument instrument : list) {
                String patchToString = this.patchToString(instrument.getPatch());
                this.inslist.put(patchToString, new SoftInstrument((ModelInstrument)instrument));
                this.loadedlist.put(patchToString, (ModelInstrument)instrument);
            }
        }
        return true;
    }
    
    private void processPropertyInfo(Map<String, Object> map) {
        AudioSynthesizerPropertyInfo[] propertyInfo = this.getPropertyInfo(map);
        String s = (String)propertyInfo[0].value;
        if (s.equalsIgnoreCase("point")) {
            this.resampler = new SoftPointResampler();
            this.resamplerType = "point";
        }
        else if (s.equalsIgnoreCase("linear")) {
            this.resampler = new SoftLinearResampler2();
            this.resamplerType = "linear";
        }
        else if (s.equalsIgnoreCase("linear1")) {
            this.resampler = new SoftLinearResampler();
            this.resamplerType = "linear1";
        }
        else if (s.equalsIgnoreCase("linear2")) {
            this.resampler = new SoftLinearResampler2();
            this.resamplerType = "linear2";
        }
        else if (s.equalsIgnoreCase("cubic")) {
            this.resampler = new SoftCubicResampler();
            this.resamplerType = "cubic";
        }
        else if (s.equalsIgnoreCase("lanczos")) {
            this.resampler = new SoftLanczosResampler();
            this.resamplerType = "lanczos";
        }
        else if (s.equalsIgnoreCase("sinc")) {
            this.resampler = new SoftSincResampler();
            this.resamplerType = "sinc";
        }
        this.setFormat((AudioFormat)propertyInfo[2].value);
        this.controlrate = (float)propertyInfo[1].value;
        this.latency = (long)propertyInfo[3].value;
        this.deviceid = (int)propertyInfo[4].value;
        this.maxpoly = (int)propertyInfo[5].value;
        this.reverb_on = (boolean)propertyInfo[6].value;
        this.chorus_on = (boolean)propertyInfo[7].value;
        this.agc_on = (boolean)propertyInfo[8].value;
        this.largemode = (boolean)propertyInfo[9].value;
        this.number_of_midi_channels = (int)propertyInfo[10].value;
        this.jitter_correction = (boolean)propertyInfo[11].value;
        this.reverb_light = (boolean)propertyInfo[12].value;
        this.load_default_soundbank = (boolean)propertyInfo[13].value;
    }
    
    private String patchToString(Patch patch) {
        if (patch instanceof ModelPatch && ((ModelPatch)patch).isPercussion()) {
            return "p." + patch.getProgram() + "." + patch.getBank();
        }
        return patch.getProgram() + "." + patch.getBank();
    }
    
    private void setFormat(AudioFormat format) {
        if (format.getChannels() > 2) {
            throw new IllegalArgumentException("Only mono and stereo audio supported.");
        }
        if (AudioFloatConverter.getConverter(format) == null) {
            throw new IllegalArgumentException("Audio format not supported.");
        }
        this.format = format;
        return;
    }
    
    void removeReceiver(Receiver o) {
        boolean n = false;
        synchronized (this.control_mutex) {
            if (this.recvslist.remove(o) && this.implicitOpen && this.recvslist.isEmpty()) {
                n = true;
            }
        }
        if (!(n == 0)) {
            this.close();
        }
    }
    
    SoftMainMixer getMainMixer() {
        if (!this.isOpen()) {
            return null;
        }
        return this.mainmixer;
    }
    
    SoftInstrument findInstrument(int i, int n, int n2) {
        if (n >> 7 == 120 || n >> 7 == 121) {
            SoftInstrument softInstrument4 = (SoftInstrument)this.inslist.get(i + "." + n);
            if (softInstrument4 != null) {
                return softInstrument4;
            }
            String s;
            if (n >> 7 == 120) {
                s = "p.";
            }
            else {
                s = "";
            }
            SoftInstrument softInstrument5 = (SoftInstrument)this.inslist.get(s + i + "." + ((n & 0x80) << 7));
            if (softInstrument5 != null) {
                return softInstrument5;
            }
            SoftInstrument softInstrument6 = (SoftInstrument)this.inslist.get(s + i + "." + (n & 0x80));
            if (softInstrument6 != null) {
                return softInstrument6;
            }
            SoftInstrument softInstrument7 = (SoftInstrument)this.inslist.get(s + i + ".0");
            if (softInstrument7 != null) {
                return softInstrument7;
            }
            SoftInstrument softInstrument8 = (SoftInstrument)this.inslist.get(s + i + "0.0");
            if (softInstrument8 != null) {
                return softInstrument8;
            }
            return null;
        }
        else {
            String str;
            if (n2 == 9) {
                str = "p.";
            }
            else {
                str = "";
            }
            SoftInstrument softInstrument = (SoftInstrument)this.inslist.get(str + i + "." + n);
            if (softInstrument != null) {
                return softInstrument;
            }
            SoftInstrument softInstrument2 = (SoftInstrument)this.inslist.get(str + i + ".0");
            if (softInstrument2 != null) {
                return softInstrument2;
            }
            SoftInstrument softInstrument3 = (SoftInstrument)this.inslist.get(str + "0.0");
            if (softInstrument3 != null) {
                return softInstrument3;
            }
            return null;
        }
    }
    
    int getVoiceAllocationMode() {
        return this.voice_allocation_mode;
    }
    
    int getGeneralMidiMode() {
        return this.gmmode;
    }
    
    void setGeneralMidiMode(int gmmode) {
        this.gmmode = gmmode;
    }
    
    int getDeviceID() {
        return this.deviceid;
    }
    
    float getControlRate() {
        return this.controlrate;
    }
    
    SoftVoice[] getVoices() {
        return this.voices;
    }
    
    SoftTuning getTuning(Patch patch) {
        String patchToString = this.patchToString(patch);
        SoftTuning softTuning = (SoftTuning)this.tunings.get(patchToString);
        if (softTuning == null) {
            softTuning = new SoftTuning(patch);
            this.tunings.put(patchToString, softTuning);
            return softTuning;
        }
        return softTuning;
    }
    
    @Override
    public long getLatency() {
        synchronized (this.control_mutex) {
            return this.latency;
        }
    }
    
    @Override
    public AudioFormat getFormat() {
        synchronized (this.control_mutex) {
            return this.format;
        }
    }
    
    @Override
    public int getMaxPolyphony() {
        synchronized (this.control_mutex) {
            return this.maxpoly;
        }
    }
    
    @Override
    public MidiChannel[] getChannels() {
        synchronized (this.control_mutex) {
            if (this.external_channels == null) {
                this.external_channels = new SoftChannelProxy[16];
                for (int i = 0; i < this.external_channels.length; ++i) {
                    this.external_channels[i] = new SoftChannelProxy();
                }
            }
            MidiChannel[] array;
            if (this.isOpen()) {
                array = new MidiChannel[this.channels.length];
            }
            else {
                array = new MidiChannel[16];
            }
            for (int j = 0; j < array.length; ++j) {
                array[j] = this.external_channels[j];
            }
            return array;
        }
    }
    
    @Override
    public VoiceStatus[] getVoiceStatus() {
        if (!this.isOpen()) {
            VoiceStatus[] array3 = new VoiceStatus[this.getMaxPolyphony()];
            for (int j = 0; j < array3.length; ++j) {
                VoiceStatus o2 = new VoiceStatus();
                o2.active = false;
                o2.bank = 0;
                o2.channel = 0;
                o2.note = 0;
                o2.program = 0;
                o2.volume = 0;
                array3[j] = o2;
            }
            return array3;
        }
        synchronized (this.control_mutex) {
            VoiceStatus[] array = new VoiceStatus[this.voices.length];
            for (int i = 0; i < this.voices.length; ++i) {
                SoftVoice softVoice = this.voices[i];
                VoiceStatus o = new VoiceStatus();
                o.active = softVoice.active;
                o.bank = softVoice.bank;
                o.channel = softVoice.channel;
                o.note = softVoice.note;
                o.program = softVoice.program;
                o.volume = softVoice.volume;
                array[i] = o;
            }
            return array;
        }
    }
    
    @Override
    public boolean isSoundbankSupported(Soundbank soundbank) {
        Instrument[] instruments = soundbank.getInstruments();
        int length = instruments.length;
        int i = 0;
        while (i < length) {
            if (!(instruments[i] instanceof ModelInstrument)) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    @Override
    public boolean loadInstrument(Instrument obj) {
        if (obj == null || !(obj instanceof ModelInstrument)) {
            throw new IllegalArgumentException("Unsupported instrument: " + obj);
        }
        ArrayList list = new ArrayList();
        list.add((Instrument)obj);
        return this.loadInstruments(list);
    }
    
    @Override
    public void unloadInstrument(Instrument obj) {
        if (obj == null || !(obj instanceof ModelInstrument)) {
            throw new IllegalArgumentException("Unsupported instrument: " + obj);
        }
        if (!this.isOpen()) {
            return;
        }
        String patchToString = this.patchToString(obj.getPatch());
        synchronized (this.control_mutex) {
            SoftChannel[] channels = this.channels;
            for (int length = channels.length, i = 0; i < length; ++i) {
                channels[i].current_instrument = null;
            }
            this.inslist.remove(patchToString);
            this.loadedlist.remove(patchToString);
            for (int j = 0; j < this.channels.length; ++j) {
                this.channels[j].allSoundOff();
            }
        }
        return;
    }
    
    @Override
    public boolean remapInstrument(Instrument instrument, Instrument instrument2) {
        if (instrument == null) {
            throw new NullPointerException();
        }
        if (instrument2 == null) {
            throw new NullPointerException();
        }
        if (!(instrument instanceof ModelInstrument)) {
            throw new IllegalArgumentException("Unsupported instrument: " + instrument.toString());
        }
        if (!(instrument2 instanceof ModelInstrument)) {
            throw new IllegalArgumentException("Unsupported instrument: " + instrument2.toString());
        }
        if (!this.isOpen()) {
            return false;
        }
        synchronized (this.control_mutex) {
            if (!this.loadedlist.containsValue(instrument2)) {
                throw new IllegalArgumentException("Instrument to is not loaded.");
            }
            this.unloadInstrument(instrument);
            return this.loadInstrument(new ModelMappedInstrument((ModelInstrument)instrument2, instrument.getPatch()));
        }
    }
    
    @Override
    public Soundbank getDefaultSoundbank() {
        synchronized (SoftSynthesizer.class) {
            if (SoftSynthesizer.defaultSoundBank != null) {
                return SoftSynthesizer.defaultSoundBank;
            }
            ArrayList list = new ArrayList();
            list.add(new PrivilegedAction<InputStream>() {
                @Override
                public InputStream run() {
                    File file = new File(new File(new File(System.getProperties().getProperty("java.home")), "lib"), "audio");
                    if (file.exists()) {
                        File file2 = null;
                        File[] listFiles = file.listFiles();
                        if (listFiles != null) {
                            for (int i = 0; i < listFiles.length; ++i) {
                                File file3 = listFiles[i];
                                if (file3.isFile()) {
                                    String lowerCase = file3.getName().toLowerCase();
                                    if ((lowerCase.endsWith(".sf2") || lowerCase.endsWith(".dls")) && (file2 == null || file3.length() > file2.length())) {
                                        file2 = file3;
                                    }
                                }
                            }
                        }
                        if (file2 != null) {
                            try {
                                return new FileInputStream(file2);
                            }
                            catch (IOException ex) {}
                        }
                    }
                    return null;
                }
            });
            list.add(new PrivilegedAction<InputStream>() {
                @Override
                public InputStream run() {
                    if (System.getProperties().getProperty("os.name").startsWith("Linux")) {
                        File[] array;
                        for (File parent : array = new File[] { new File("/usr/share/soundfonts/"), new File("/usr/local/share/soundfonts/"), new File("/usr/share/sounds/sf2/"), new File("/usr/local/share/sounds/sf2/") }) {
                            if (parent.exists()) {
                                File file = new File(parent, "default.sf2");
                                if (file.exists()) {
                                    try {
                                        return new FileInputStream(file);
                                    }
                                    catch (IOException ex) {}
                                }
                            }
                        }
                    }
                    return null;
                }
            });
            list.add(new PrivilegedAction<InputStream>() {
                @Override
                public InputStream run() {
                    if (System.getProperties().getProperty("os.name").startsWith("Windows")) {
                        File file = new File(System.getenv("SystemRoot") + "\\system32\\drivers\\gm.dls");
                        if (file.exists()) {
                            try {
                                return new FileInputStream(file);
                            }
                            catch (IOException ex) {}
                        }
                    }
                    return null;
                }
            });
            list.add(new PrivilegedAction<InputStream>() {
                @Override
                public InputStream run() {
                    File file = new File(new File(System.getProperty("user.home"), ".gervill"), "soundbank-emg.sf2");
                    if (file.exists()) {
                        try {
                            return new FileInputStream(file);
                        }
                        catch (IOException ex) {}
                    }
                    return null;
                }
            });
            for (PrivilegedAction action : list) {
                try {
                    InputStream in = (InputStream)AccessController.doPrivileged(action);
                    if (in == null) {
                        continue;
                    }
                    Soundbank soundbank;
                    try {
                        soundbank = MidiSystem.getSoundbank(new BufferedInputStream(in));
                    }
                    finally {
                        in.close();
                    }
                    if (soundbank != null) {
                        SoftSynthesizer.defaultSoundBank = soundbank;
                        return SoftSynthesizer.defaultSoundBank;
                    }
                    continue;
                }
                catch (Exception ex) {}
            }
            try {
                SoftSynthesizer.defaultSoundBank = EmergencySoundbank.createSoundbank();
            }
            catch (Exception ex2) {}
            if (SoftSynthesizer.defaultSoundBank != null) {
                OutputStream outputStream = (OutputStream)AccessController.doPrivileged(()/*  */ -> {
                    try {
                        File parent = new File(System.getProperty("user.home"), ".gervill");
                        if (!parent.exists()) {
                            parent.mkdirs();
                        }
                        File file = new File(parent, "soundbank-emg.sf2");
                        if (file.exists()) {
                            return null;
                        }
                        return new FileOutputStream(file);
                    }
                    catch (FileNotFoundException ex4) {
                        return null;
                    }
                });
                if (outputStream != null) {
                    try {
                        ((SF2Soundbank)SoftSynthesizer.defaultSoundBank).save(outputStream);
                        outputStream.close();
                    }
                    catch (IOException ex3) {}
                }
            }
        }
        return SoftSynthesizer.defaultSoundBank;
    }
    
    @Override
    public Instrument[] getAvailableInstruments() {
        Soundbank defaultSoundbank = this.getDefaultSoundbank();
        if (defaultSoundbank == null) {
            return new Instrument[0];
        }
        Instrument[] instruments = defaultSoundbank.getInstruments();
        Arrays.sort(instruments, new ModelInstrumentComparator());
        return instruments;
    }
    
    @Override
    public Instrument[] getLoadedInstruments() {
        if (!this.isOpen()) {
            return new Instrument[0];
        }
        synchronized (this.control_mutex) {
            ModelInstrument[] a = new ModelInstrument[this.loadedlist.values().size()];
            this.loadedlist.values().toArray(a);
            Arrays.sort(a, new ModelInstrumentComparator());
            return a;
        }
    }
    
    @Override
    public boolean loadAllInstruments(Soundbank soundbank) {
        ArrayList<ModelInstrument> list = new ArrayList();
        Instrument[] instruments = soundbank.getInstruments();
        int length = instruments.length;
        int i = 0;
        while (i < length) {
            Instrument obj = instruments[i];
            if (obj == null || !(obj instanceof ModelInstrument)) {
                throw new IllegalArgumentException("Unsupported instrument: " + obj);
            }
            list.add((ModelInstrument)obj);
            ++i;
        }
        return this.loadInstruments(list);
    }
    
    @Override
    public void unloadAllInstruments(Soundbank obj) {
        if (obj == null || !this.isSoundbankSupported(obj)) {
            throw new IllegalArgumentException("Unsupported soundbank: " + obj);
        }
        if (!this.isOpen()) {
            return;
        }
        for (Instrument instrument : obj.getInstruments()) {
            if ((instrument instanceof ModelInstrument)) {
                this.unloadInstrument(instrument);
            }
        }
        return;
    }
    
    @Override
    public boolean loadInstruments(Soundbank soundbank, Patch[] array) {
        ArrayList<ModelInstrument> list = new ArrayList();
        int length = array.length;
        int i = 0;
        while (i < length) {
            Instrument instrument = soundbank.getInstrument(array[i]);
            if (instrument == null || !(instrument instanceof ModelInstrument)) {
                throw new IllegalArgumentException("Unsupported instrument: " + instrument);
            }
            list.add((ModelInstrument)instrument);
            ++i;
        }
        return this.loadInstruments(list);
    }
    
    @Override
    public void unloadInstruments(Soundbank obj, Patch[] array) {
        if (obj == null || !this.isSoundbankSupported(obj)) {
            throw new IllegalArgumentException("Unsupported soundbank: " + obj);
        }
        if (!this.isOpen()) {
            return;
        }
        for (int length = array.length, i = 0; i < length; ++i) {
            Instrument instrument = obj.getInstrument(array[i]);
            if ((instrument instanceof ModelInstrument)) {
                this.unloadInstrument(instrument);
            }
        }
        return;
    }
    
    @Override
    public MidiDevice.Info getDeviceInfo() {
        return SoftSynthesizer.info;
    }
    
    private Properties getStoredProperties() {
        return (Properties)AccessController.doPrivileged(()/*  */ -> {
            Properties properties = new Properties();
            try {
                Preferences preferences = Preferences.userRoot();
                String s;
                if (preferences.nodeExists(s)) {
                    Preferences preferences2 = preferences.node(s);
                    String[] array = preferences2.keys();
                    int i;
                    for (int length = array.length; i < length; ++i) {
                        String key = array[i];
                        String value = preferences2.get(key, null);
                        if (value != null) {
                            properties.setProperty(key, value);
                        }
                    }
                }
            }
            catch (BackingStoreException ex) {}
            return properties;
        });
    }
    
    @Override
    public AudioSynthesizerPropertyInfo[] getPropertyInfo(Map<String, Object> map) {
        ArrayList<Object> list = new ArrayList();
        int n = map == null && this.open;
        AudioSynthesizerPropertyInfo o = new AudioSynthesizerPropertyInfo("interpolation", n ? this.resamplerType : "linear");
        o.choices = new String[] { "linear", "linear1", "linear2", "cubic", "lanczos", "sinc", "point" };
        o.description = "Interpolation method";
        list.add(o);
        AudioSynthesizerPropertyInfo o2 = new AudioSynthesizerPropertyInfo("control rate", Float.valueOf(n ? this.controlrate : 147.0f));
        o2.description = "Control rate";
        list.add(o2);
        AudioSynthesizerPropertyInfo o3 = new AudioSynthesizerPropertyInfo("format", n ? this.format : new AudioFormat(44100.0f, 16, 2, true, false));
        o3.description = "Default audio format";
        list.add(o3);
        AudioSynthesizerPropertyInfo o4 = new AudioSynthesizerPropertyInfo("latency", Long.valueOf(n ? this.latency : 120000L));
        o4.description = "Default latency";
        list.add(o4);
        AudioSynthesizerPropertyInfo o5 = new AudioSynthesizerPropertyInfo("device id", Integer.valueOf(n ? this.deviceid : 0));
        o5.description = "Device ID for SysEx Messages";
        list.add(o5);
        AudioSynthesizerPropertyInfo o6 = new AudioSynthesizerPropertyInfo("max polyphony", Integer.valueOf(n ? this.maxpoly : 64));
        o6.description = "Maximum polyphony";
        list.add(o6);
        AudioSynthesizerPropertyInfo o7 = new AudioSynthesizerPropertyInfo("reverb", Boolean.valueOf(n == 0 || this.reverb_on));
        o7.description = "Turn reverb effect on or off";
        list.add(o7);
        AudioSynthesizerPropertyInfo o8 = new AudioSynthesizerPropertyInfo("chorus", Boolean.valueOf(n == 0 || this.chorus_on));
        o8.description = "Turn chorus effect on or off";
        list.add(o8);
        AudioSynthesizerPropertyInfo o9 = new AudioSynthesizerPropertyInfo("auto gain control", Boolean.valueOf(n == 0 || this.agc_on));
        o9.description = "Turn auto gain control on or off";
        list.add(o9);
        AudioSynthesizerPropertyInfo o10 = new AudioSynthesizerPropertyInfo("large mode", Boolean.valueOf(n && this.largemode));
        o10.description = "Turn large mode on or off.";
        list.add(o10);
        AudioSynthesizerPropertyInfo o11 = new AudioSynthesizerPropertyInfo("midi channels", Integer.valueOf(n ? this.channels.length : 16));
        o11.description = "Number of midi channels.";
        list.add(o11);
        AudioSynthesizerPropertyInfo o12 = new AudioSynthesizerPropertyInfo("jitter correction", Boolean.valueOf(n == 0 || this.jitter_correction));
        o12.description = "Turn jitter correction on or off.";
        list.add(o12);
        AudioSynthesizerPropertyInfo o13 = new AudioSynthesizerPropertyInfo("light reverb", Boolean.valueOf(n == 0 || this.reverb_light));
        o13.description = "Turn light reverb mode on or off";
        list.add(o13);
        AudioSynthesizerPropertyInfo o14 = new AudioSynthesizerPropertyInfo("load default soundbank", Boolean.valueOf(n == 0 || this.load_default_soundbank));
        o14.description = "Enabled/disable loading default soundbank";
        list.add(o14);
        AudioSynthesizerPropertyInfo[] array = (AudioSynthesizerPropertyInfo[])list.toArray(new AudioSynthesizerPropertyInfo[list.size()]);
        Properties storedProperties = this.getStoredProperties();
        for (AudioSynthesizerPropertyInfo audioSynthesizerPropertyInfo : array) {
            Object o15 = (map == null) ? null : map.get(audioSynthesizerPropertyInfo.name);
            Object value = (o15 != null) ? o15 : storedProperties.getProperty(audioSynthesizerPropertyInfo.name);
            if (value != null) {
                Class<Object> valueClass = audioSynthesizerPropertyInfo.valueClass;
                if (valueClass.isInstance(value)) {
                    audioSynthesizerPropertyInfo.value = value;
                }
                else if ((value instanceof String)) {
                    String str = (String)value;
                    if (valueClass == Boolean.class) {
                        if (str.equalsIgnoreCase("true")) {
                            audioSynthesizerPropertyInfo.value = Boolean.TRUE;
                        }
                        if (str.equalsIgnoreCase("false")) {
                            audioSynthesizerPropertyInfo.value = Boolean.FALSE;
                        }
                    }
                    else if (valueClass == AudioFormat.class) {
                        int int1 = 2;
                        boolean signed = true;
                        boolean bigEndian = false;
                        int int2 = 16;
                        float float1 = 44100.0f;
                        try {
                            StringTokenizer o16 = new StringTokenizer(str, ", ");
                            String s = "";
                            while (o16.hasMoreTokens()) {
                                String lowerCase = o16.nextToken().toLowerCase();
                                if (lowerCase.equals("mono")) {
                                    int1 = 1;
                                }
                                if (lowerCase.startsWith("channel")) {
                                    int1 = Integer.parseInt(s);
                                }
                                if (lowerCase.contains("unsigned")) {
                                    signed = false;
                                }
                                if (lowerCase.equals("big-endian")) {
                                    bigEndian = true;
                                }
                                if (lowerCase.equals("bit")) {
                                    int2 = Integer.parseInt(s);
                                }
                                if (lowerCase.equals("hz")) {
                                    float1 = Float.parseFloat(s);
                                }
                                s = lowerCase;
                            }
                            audioSynthesizerPropertyInfo.value = new AudioFormat(float1, int2, int1, (boolean)signed, (boolean)bigEndian);
                        }
                        catch (NumberFormatException ex2) {}
                    }
                    else {
                        try {
                            if (valueClass == Byte.class) {
                                audioSynthesizerPropertyInfo.value = Byte.valueOf(str);
                            }
                            else if (valueClass == Short.class) {
                                audioSynthesizerPropertyInfo.value = Short.valueOf(str);
                            }
                            else if (valueClass == Integer.class) {
                                audioSynthesizerPropertyInfo.value = Integer.valueOf(str);
                            }
                            else if (valueClass == Long.class) {
                                audioSynthesizerPropertyInfo.value = Long.valueOf(str);
                            }
                            else if (valueClass == Float.class) {
                                audioSynthesizerPropertyInfo.value = Float.valueOf(str);
                            }
                            else if (valueClass == Double.class) {
                                audioSynthesizerPropertyInfo.value = Double.valueOf(str);
                            }
                        }
                        catch (NumberFormatException ex) {}
                    }
                }
                else if ((value instanceof Number)) {
                    Number n2 = (Number)value;
                    if (valueClass == Byte.class) {
                        audioSynthesizerPropertyInfo.value = Byte.valueOf(n2.byteValue());
                    }
                    if (valueClass == Short.class) {
                        audioSynthesizerPropertyInfo.value = Short.valueOf(n2.shortValue());
                    }
                    if (valueClass == Integer.class) {
                        audioSynthesizerPropertyInfo.value = Integer.valueOf(n2.intValue());
                    }
                    if (valueClass == Long.class) {
                        audioSynthesizerPropertyInfo.value = Long.valueOf(n2.longValue());
                    }
                    if (valueClass == Float.class) {
                        audioSynthesizerPropertyInfo.value = Float.valueOf(n2.floatValue());
                    }
                    if (valueClass == Double.class) {
                        audioSynthesizerPropertyInfo.value = Double.valueOf(n2.doubleValue());
                    }
                }
            }
        }
        return array;
    }
    
    @Override
    public void open() throws MidiUnavailableException {
        if (this.isOpen()) {
            synchronized (this.control_mutex) {
                this.implicitOpen = false;
            }
            return;
        }
        this.open(null, null);
        return;
    }
    
    @Override
    public void open(SourceDataLine sourceDataLine, Map<String, Object> map) throws MidiUnavailableException {
        if (this.isOpen()) {
            synchronized (this.control_mutex) {
                this.implicitOpen = false;
            }
            return;
        }
        synchronized (this.control_mutex) {
            try {
                if (sourceDataLine != null) {
                    this.setFormat(sourceDataLine.getFormat());
                }
                this.weakstream = new SoftSynthesizer.WeakAudioStream(this.openStream(this.getFormat(), map));
                AudioInputStream audioInputStream = this.weakstream.getAudioInputStream();
                if (sourceDataLine == null) {
                    if (SoftSynthesizer.testline != null) {
                        sourceDataLine = SoftSynthesizer.testline;
                    }
                    else {
                        sourceDataLine = AudioSystem.getSourceDataLine(this.getFormat());
                    }
                }
                double n = (double)this.latency;
                if (!sourceDataLine.isOpen()) {
                    sourceDataLine.open(this.getFormat(), this.getFormat().getFrameSize() * (int)((double)this.getFormat().getFrameRate() * (n / 1000000.0)));
                    this.sourceDataLine = sourceDataLine;
                }
                if (!sourceDataLine.isActive()) {
                    sourceDataLine.start();
                }
                int available = 512;
                try {
                    available = audioInputStream.available();
                }
                catch (IOException ex) {}
                int bufferSize = sourceDataLine.getBufferSize();
                int n2 = bufferSize - bufferSize % available;
                if (n2 < 3 * available) {
                    n2 = 3 * available;
                }
                if (this.jitter_correction) {
                    audioInputStream = new SoftJitterCorrector(audioInputStream, n2, available);
                    if (this.weakstream != null) {
                        this.weakstream.jitter_stream = audioInputStream;
                    }
                }
                this.pusher = new SoftAudioPusher(sourceDataLine, audioInputStream, available);
                this.pusher_stream = audioInputStream;
                this.pusher.start();
                if (this.weakstream != null) {
                    this.weakstream.pusher = this.pusher;
                    this.weakstream.sourceDataLine = this.sourceDataLine;
                }
            }
            catch (LineUnavailableException | SecurityException | IllegalArgumentException cause) {
                if (this.isOpen()) {
                    this.close();
                }
                MidiUnavailableException ex2 = new MidiUnavailableException("Can not open line");
                ex2.initCause(cause);
                throw ex2;
            }
        }
        return;
    }
    
    @Override
    public AudioInputStream openStream(AudioFormat format, Map<String, Object> map) throws MidiUnavailableException {
        if (this.isOpen()) {
            throw new MidiUnavailableException("Synthesizer is already open");
        }
        synchronized (this.control_mutex) {
            this.gmmode = 0;
            this.voice_allocation_mode = 0;
            this.processPropertyInfo(map);
            this.open = true;
            this.implicitOpen = false;
            if (format != null) {
                this.setFormat(format);
            }
            if (this.load_default_soundbank) {
                Soundbank defaultSoundbank = this.getDefaultSoundbank();
                if (defaultSoundbank != null) {
                    this.loadAllInstruments(defaultSoundbank);
                }
            }
            this.voices = new SoftVoice[this.maxpoly];
            for (int i = 0; i < this.maxpoly; ++i) {
                this.voices[i] = new SoftVoice(this);
            }
            this.mainmixer = new SoftMainMixer(this);
            this.channels = new SoftChannel[this.number_of_midi_channels];
            for (int j = 0; j < this.channels.length; ++j) {
                this.channels[j] = new SoftChannel(this, j);
            }
            if (this.external_channels == null) {
                if (this.channels.length < 16) {
                    this.external_channels = new SoftChannelProxy[16];
                }
                else {
                    this.external_channels = new SoftChannelProxy[this.channels.length];
                }
                for (int n = 0; n < this.external_channels.length; ++n) {
                    this.external_channels[n] = new SoftChannelProxy();
                }
            }
            else if (this.channels.length > this.external_channels.length) {
                SoftChannelProxy[] array = new SoftChannelProxy[this.channels.length];
                for (int k = 0; k < this.external_channels.length; ++k) {
                    array[k] = this.external_channels[k];
                }
                for (int l = this.external_channels.length; l < array.length; ++l) {
                    array[l] = new SoftChannelProxy();
                }
            }
            for (int n2 = 0; n2 < this.channels.length; ++n2) {
                this.external_channels[n2].setChannel(this.channels[n2]);
            }
            SoftVoice[] voices = this.getVoices();
            for (int length = voices.length, n3 = 0; n3 < length; ++n3) {
                voices[n3].resampler = this.resampler.openStreamer();
            }
            Iterator<Receiver> iterator = this.getReceivers().iterator();
            while (iterator.hasNext()) {
                SoftReceiver softReceiver = (SoftReceiver)(Receiver)iterator.next();
                softReceiver.open = this.open;
                softReceiver.mainmixer = this.mainmixer;
                softReceiver.midimessages = this.mainmixer.midimessages;
            }
            return this.mainmixer.getInputStream();
        }
    }
    
    @Override
    public void close() {
        if (!this.isOpen()) {
            return;
        }
        SoftAudioPusher pusher = null;
        AudioInputStream pusher_stream = null;
        synchronized (this.control_mutex) {
            if (this.pusher != null) {
                pusher = this.pusher;
                pusher_stream = this.pusher_stream;
                this.pusher = null;
                this.pusher_stream = null;
            }
        }
        if (pusher != null) {
            pusher.stop();
            try {
                pusher_stream.close();
            }
            catch (IOException ex) {}
        }
        synchronized (this.control_mutex) {
            if (this.mainmixer != null) {
                this.mainmixer.close();
            }
            this.open = false;
            this.implicitOpen = false;
            this.mainmixer = null;
            this.voices = null;
            this.channels = null;
            if (this.external_channels != null) {
                for (int i = 0; i < this.external_channels.length; ++i) {
                    this.external_channels[i].setChannel(null);
                }
            }
            if (this.sourceDataLine != null) {
                this.sourceDataLine.close();
                this.sourceDataLine = null;
            }
            this.inslist.clear();
            this.loadedlist.clear();
            this.tunings.clear();
            while (this.recvslist.size() != 0) {
                ((Receiver)this.recvslist.get(this.recvslist.size() - 1)).close();
            }
        }
        return;
    }
    
    @Override
    public boolean isOpen() {
        synchronized (this.control_mutex) {
            return this.open;
        }
    }
    
    @Override
    public long getMicrosecondPosition() {
        if (!this.isOpen()) {
            return 0L;
        }
        synchronized (this.control_mutex) {
            return this.mainmixer.getMicrosecondPosition();
        }
    }
    
    @Override
    public int getMaxReceivers() {
        return -1;
    }
    
    @Override
    public int getMaxTransmitters() {
        return 0;
    }
    
    @Override
    public Receiver getReceiver() throws MidiUnavailableException {
        synchronized (this.control_mutex) {
            SoftReceiver e = new SoftReceiver(this);
            e.open = this.open;
            this.recvslist.add(e);
            return e;
        }
    }
    
    @Override
    public List<Receiver> getReceivers() {
        synchronized (this.control_mutex) {
            ArrayList<Object> list = new ArrayList();
            list.addAll(this.recvslist);
            return list;
        }
    }
    
    @Override
    public Transmitter getTransmitter() throws MidiUnavailableException {
        throw new MidiUnavailableException("No transmitter available");
    }
    
    @Override
    public List<Transmitter> getTransmitters() {
        return new ArrayList();
    }
    
    @Override
    public Receiver getReceiverReferenceCounting() throws MidiUnavailableException {
        if (!this.isOpen()) {
            this.open();
            synchronized (this.control_mutex) {
                this.implicitOpen = true;
            }
        }
        return this.getReceiver();
    }
    
    @Override
    public Transmitter getTransmitterReferenceCounting() throws MidiUnavailableException {
        throw new MidiUnavailableException("No transmitter available");
    }
    
    static {
        info = new SoftSynthesizer.Info();
        SoftSynthesizer.testline = null;
        SoftSynthesizer.defaultSoundBank = null;
    }
    
    protected static final class WeakAudioStream extends InputStream
    {
        private volatile AudioInputStream stream;
        public SoftAudioPusher pusher;
        public AudioInputStream jitter_stream;
        public SourceDataLine sourceDataLine;
        public volatile long silent_samples;
        private int framesize;
        private WeakReference<AudioInputStream> weak_stream_link;
        private AudioFloatConverter converter;
        private float[] silentbuffer;
        private int samplesize;
        
        public void setInputStream(AudioInputStream stream) {
            this.stream = stream;
        }
        
        @Override
        public int available() throws IOException {
            AudioInputStream stream = this.stream;
            if (stream != null) {
                return stream.available();
            }
            return 0;
        }
        
        @Override
        public int read() throws IOException {
            byte[] b = { 0 };
            if (super.read(b) == -1) {
                return -1;
            }
            return b[0] & 0xFF;
        }
        
        @Override
        public int read(byte[] b, int off, int len) throws IOException {
            AudioInputStream stream = this.stream;
            if (stream != null) {
                return stream.read(b, off, len);
            }
            int n = len / this.samplesize;
            if (this.silentbuffer == null || this.silentbuffer.length < n) {
                this.silentbuffer = new float[n];
            }
            this.converter.toByteArray(this.silentbuffer, n, b, off);
            this.silent_samples += (long)(len / this.framesize);
            if (this.pusher != null && this.weak_stream_link.get() == null) {
                Runnable task = new Runnable() {
                    SoftAudioPusher _pusher = SoftSynthesizer.WeakAudioStream.this.pusher;
                    AudioInputStream _jitter_stream = SoftSynthesizer.WeakAudioStream.this.jitter_stream;
                    SourceDataLine _sourceDataLine = SoftSynthesizer.WeakAudioStream.this.sourceDataLine;
                    
                    @Override
                    public void run() {
                        this._pusher.stop();
                        if (this._jitter_stream != null) {
                            try {
                                this._jitter_stream.close();
                            }
                            catch (IOException ex) {
                                ex.printStackTrace();
                            }
                        }
                        if (this._sourceDataLine != null) {
                            this._sourceDataLine.close();
                        }
                    }
                };
                this.pusher = null;
                this.jitter_stream = null;
                this.sourceDataLine = null;
                new Thread(task).start();
            }
            return len;
        }
        
        public WeakAudioStream(AudioInputStream audioInputStream) {
            this.pusher = null;
            this.jitter_stream = null;
            this.sourceDataLine = null;
            this.silent_samples = 0L;
            this.framesize = 0;
            this.silentbuffer = null;
            this.stream = audioInputStream;
            this.weak_stream_link = new WeakReference(audioInputStream);
            this.converter = AudioFloatConverter.getConverter(audioInputStream.getFormat());
            this.samplesize = audioInputStream.getFormat().getFrameSize() / audioInputStream.getFormat().getChannels();
            this.framesize = audioInputStream.getFormat().getFrameSize();
        }
        
        public AudioInputStream getAudioInputStream() {
            return new AudioInputStream(this, this.stream.getFormat(), -1L);
        }
        
        @Override
        public void close() throws IOException {
            AudioInputStream audioInputStream = (AudioInputStream)this.weak_stream_link.get();
            if (audioInputStream != null) {
                audioInputStream.close();
            }
        }
    }
    
    private static class Info extends MidiDevice.Info
    {
        Info() {
            super("Gervill", "OpenJDK", "Software MIDI Synthesizer", "1.0");
        }
    }
}
